package mvd.jester.model;

import java.lang.reflect.Type;
import java.util.TreeSet;
import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import mvd.jester.priority.PriorityManager;
import mvd.jester.priority.RateMonotonic;

/**
 * SortedTaskSet
 */
public class SortedTaskSet extends TreeSet<Task> {

    private static final long serialVersionUID = 4808544133562675597L;

    public SortedTaskSet(PriorityManager priorityMananger) {
        super((t1, t2) -> priorityMananger.compare(t1, t2));
    }

    public static class Deserializer implements JsonDeserializer<SortedTaskSet> {

        @Override
        public SortedTaskSet deserialize(JsonElement json, Type typeOfT,
                JsonDeserializationContext context) throws JsonParseException {
            SortedTaskSet taskSet = new SortedTaskSet(new RateMonotonic());
            if (json.isJsonArray()) {
                JsonArray array = json.getAsJsonArray();
                array.forEach(e -> {
                    taskSet.add(context.deserialize(e, Task.class));
                });
            }
            return taskSet;
        }
    }
}
