package mvd.jester.model;

import java.util.Set;

/**
 * Task
 */
public class Task {

    private final long deadline;
    private final long period;
    private final Set<Segment> segments;
    private final long maximumWcet;
    private final long maximumParallelism;

    public Task(long period, long deadline, Set<Segment> segments) {
        this.deadline = deadline;
        this.period = period;
        this.segments = segments;

        long maxWcet = 0;
        long maxParallelism = 0;
        for (Segment s : segments) {
            maxWcet += s.getJobWcet() * s.getNumberOfJobs();
            if (maxParallelism < s.getNumberOfJobs()) {
                maxParallelism = s.getNumberOfJobs();
            }
        }

        this.maximumParallelism = maxParallelism;
        this.maximumWcet = maxWcet;
    }

    public Task(long period, Set<Segment> segments) {
        this(period, period, segments);
    }


    /**
     * @return the deadline
     */
    public long getDeadline() {
        return deadline;
    }

    /**
     * @return the period
     */
    public long getPeriod() {
        return period;
    }

    /**
     * @return the segments
     */
    public Set<Segment> getSegments() {
        return segments;
    }

    /**
     * @return the maximumWcet
     */
    public long getMaximumWcet() {
        return maximumWcet;
    }

    /**
     * @return the maximumParallelism
     */
    public long getMaximumParallelism() {
        return maximumParallelism;
    }
}
