package mvd.jester.priority;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import mvd.jester.model.Task;
import mvd.jester.simulator.AbstractSimulator;
import mvd.jester.simulator.MaiaBertogna;
import mvd.jester.simulator.SchmidMottok;
import mvd.jester.simulator.internals.TaskContextInterface;
import mvd.jester.tests.AbstractTest;

/**
 * EarliestDeadineFirst
 */
public class EarliestDeadlineFirst implements PriorityManager {

    final static Set<Class<? extends AbstractTest>> abstractTests = new HashSet<>();
    final static Set<Class<? extends AbstractSimulator>> abstractSimulators =
            new HashSet<>(Arrays.asList(MaiaBertogna.class, SchmidMottok.class));

    /**
     * Compare the priority of two tasks according to the Rate Monotonic policy
     * 
     * @param t1 The first task
     * @param t2 The second task
     * @return 0 if both tasks have the same priority, positive number if the first task has a
     *         higher priority, negative number if the second task has a higher priority
     */
    @Override
    public int compare(Task t1, Task t2) {
        throw new RuntimeException(
                "Earliest Deadline First does not have Priority Manager for tasks");
    }

    @Override
    public int compare(TaskContextInterface t1, TaskContextInterface t2) {
        return (int) (t1.getDeadline() - t2.getDeadline());
    }

    @Override
    public boolean hasTest(AbstractTest abstractTest) {
        return abstractTests.contains(abstractTest.getClass());
    }

    @Override
    public boolean hasSimulator(AbstractSimulator abstractSimulator) {
        return abstractSimulators.contains(abstractSimulator.getClass());
    }

    @Override
    public String getName() {
        return "EDF";
    }
}
