package mvd.jester.simulator.internals;

import java.util.Optional;
import mvd.jester.simulator.EventPrinter;

/**
 * Processor
 */
public class ProcessorContext {

    private Optional<JobContextInterface> currentJob;
    private final long processorId;

    public ProcessorContext(long processorId) {
        currentJob = Optional.empty();
        this.processorId = processorId;
    }

    public void setJob(JobContextInterface job) {
        this.currentJob = Optional.ofNullable(job);
    }

    /**
     * @return the currentJob
     */
    public Optional<JobContextInterface> getJob() {
        return currentJob;
    }

    public Optional<TaskContextInterface> updateExecution(long time) {
        if (currentJob.isPresent()) {
            return currentJob.get().updateExecution(time);
        }

        return Optional.empty();
    }


    public boolean acceptTask(TaskContextInterface taskContext, int t) {
        if (!currentJob.isPresent() || currentJob.get().getTaskContext().getTask()
                .getPeriod() > taskContext.getTask().getPeriod()) {
            Optional<JobContextInterface> optionalJob = taskContext.getNextJob();

            if (optionalJob.isPresent()) {
                if (!optionalJob.get().prepareJob(t)) {
                    return false;
                }
                if (currentJob.isPresent()) {
                    currentJob.get().setCurrentProcessor(null);
                }
                currentJob = optionalJob;
                currentJob.get().setCurrentProcessor(this);
                EventPrinter.print(
                        "Time " + t + ": " + this + " started job " + currentJob.get() + "!");
                return true;
            }

        }

        return false;
    }

    @Override
    public String toString() {
        return "Processor " + processorId;
    }

}
