package mvd.jester.simulator.internals.maiabertogna;

import java.util.Optional;
import mvd.jester.simulator.EventPrinter;
import mvd.jester.simulator.internals.JobContextInterface;
import mvd.jester.simulator.internals.ProcessorContext;
import mvd.jester.simulator.internals.TaskContextInterface;

/**
 * Job
 * 
 * @param <Job>
 */
public class JobContext implements JobContextInterface {

    private final TaskContext taskContext;
    private final SegmentContext segmentContext;
    private final long wcet;
    private Optional<ProcessorContext> currentProcessor;
    private long executionTime;


    public JobContext(TaskContext taskContext, SegmentContext segmentContext) {
        this.currentProcessor = Optional.empty();
        this.taskContext = taskContext;
        this.segmentContext = segmentContext;
        this.wcet = segmentContext.getSegment().getJobWcet();
        this.executionTime = wcet;
    }

    public Optional<TaskContextInterface> updateExecution(long time) {
        executionTime--;

        if (executionTime == 0) {
            EventPrinter.print("Time " + time + ": " + currentProcessor.get()
                    + " finished execution of job " + this + "!");
            currentProcessor.get().setJob(null);
            currentProcessor = Optional.empty();
            return taskContext.acceptNotification(time);
        } else if (executionTime < 0) {
            throw new RuntimeException("Job was executed for longer than its WCET!");
        }

        return Optional.empty();
    }

    public boolean checkExecutionTime() {
        return executionTime > 0;
    }

    /**
     * @return the wcet
     */
    public long getWcet() {
        return wcet;
    }

    /**
     * @param processor the currentProcessor to set
     */
    public void setCurrentProcessor(ProcessorContext processor) {
        this.currentProcessor = Optional.ofNullable(processor);
    }

    /**
     * @return the currentProcessor
     */
    public Optional<ProcessorContext> getCurrentProcessor() {
        return currentProcessor;
    }

    /**
     * @return the segmentContext
     */
    public SegmentContext getSegmentContext() {
        return segmentContext;
    }


    /**
     * @return the taskContext
     */
    public TaskContext getTaskContext() {
        return taskContext;
    }

    @Override
    public String toString() {
        return "(jobWcet=" + wcet + ", of task=" + taskContext + ")";
    }

    @Override
    public boolean prepareJob(long time) {
        return true;
    }

}
