package mvd.jester.simulator.internals.maiabertogna;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import mvd.jester.model.Segment;
import mvd.jester.simulator.internals.JobContextInterface;

/**
 * Segment
 */
public class SegmentContext {
    private final Segment segment;
    private final Set<JobContextInterface> jobs;

    public SegmentContext(TaskContext taskContext, Segment segment) {
        this.segment = segment;
        jobs = new HashSet<>();

        for (int j = 0; j < segment.getNumberOfJobs(); ++j) {
            jobs.add(new JobContext(taskContext, this));
        }
    }


    /**
     * @return the segment
     */
    public Segment getSegment() {
        return segment;
    }

    public Optional<JobContextInterface> getNextJob() {
        return jobs.stream()
                .filter(j -> !j.getCurrentProcessor().isPresent() && j.checkExecutionTime())
                .findFirst();
    }

    @Override
    public String toString() {
        return "(nJobs=" + segment.getNumberOfJobs() + ", jobWcet=" + segment.getJobWcet() + ")";
    }
}
