package mvd.jester.simulator.internals.schmidmottok;

import java.util.Optional;

/**
 * Tasklet
 */
public class TaskletContext {

    private final TaskContext taskContext;
    private Optional<JobContext> currentJob;
    private final long wcet;
    private long executionTime;


    public TaskletContext(TaskContext taskContext, SegmentContext segment) {
        this.taskContext = taskContext;
        this.wcet = segment.getSegment().getTaskletWcet();
        this.executionTime = wcet;
        currentJob = Optional.empty();
    }


    /**
     * @return the currentJob
     */
    public Optional<JobContext> getCurrentJob() {
        return currentJob;
    }

    public void setCurrentJob(JobContext jobContext) {
        currentJob = Optional.ofNullable(jobContext);
    }

    /**
     * @return true if tasklet finished, false otherwise
     */
    public boolean updateExecution(long time) {
        executionTime--;

        if (executionTime == 0) {
            if (currentJob.isPresent()) {
                currentJob.get().setCurrentTasklet(null);
            }
            currentJob = Optional.empty();
            return true;
        } else if (executionTime < 0) {
            throw new RuntimeException("Tasklet was executed for longer than its WCET!");
        }
        return false;
    }

    public boolean checkExecutionTime() {
        return executionTime > 0;
    }

    @Override
    public String toString() {
        return "(wcet=" + wcet + ", task=" + taskContext + ")";
    }
}
