package mvd.jester.model;

import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;


public class TestSystemSetup {

    private static final int NUMBER_OF_SETS = 1000;

    @Test
    @DisplayName("Check if randomly generated task parameters lie within the specified ranges.")
    public void testRandomTaskSetGeneration() {

        for (int i = 0; i < NUMBER_OF_SETS; ++i) {
            SystemSetup taskSet = new SystemSetup.Builder().setNumberOfSegments(1, 7)
                    .setNumberOfJobs(2, 10).setPeriods(100, 1000).build();

            for (Task t : taskSet.getTasks()) {
                assertTrue(t.getPeriod() >= 100);
                assertTrue(t.getPeriod() <= 1000);
                assertTrue(t.getDeadline() == t.getPeriod());

                long maxJobWcet = t.getPeriod() / t.getSegments().size();

                for (Segment s : t.getSegments()) {
                    assertTrue(s.getJobWcet() * s.getNumberOfJobs() == s.getTaskletWcet()
                            * s.getNumberOfTasklets());
                    assertTrue(s.getJobWcet() >= 1);
                    assertTrue(s.getJobWcet() <= maxJobWcet);
                    assertTrue(s.getNumberOfJobs() >= 1);
                    assertTrue(s.getNumberOfJobs() <= 10);
                }

                assertTrue(t.getSegments().size() >= 1);
                assertTrue(t.getSegments().size() <= 7);
            }
        }

    }
}
