package mvd.jester.simulator.maiabertogna;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import java.util.Optional;
import java.util.concurrent.ThreadLocalRandom;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import mvd.jester.model.Segment;
import mvd.jester.simulator.internals.ProcessorContext;
import mvd.jester.simulator.internals.TaskContextInterface;
import mvd.jester.simulator.internals.maiabertogna.JobContext;
import mvd.jester.simulator.internals.maiabertogna.SegmentContext;
import mvd.jester.simulator.internals.maiabertogna.TaskContext;

/**
 * TestJobContext
 */
public class TestJobContext {

    @Test
    @DisplayName("Check if execution of Job is updated correctly.")
    public void testUpdateExecution() {
        for (int run = 0; run < 100; ++run) {
            long jobWcet = ThreadLocalRandom.current().nextLong(20, 50);

            Segment s = mock(Segment.class);
            when(s.getJobWcet()).thenReturn(jobWcet);
            SegmentContext sc = mock(SegmentContext.class);
            when(sc.getSegment()).thenReturn(s);
            TaskContext tc = mock(TaskContext.class);
            when(tc.acceptNotification(anyLong())).thenReturn(Optional.of(tc));

            JobContext job = new JobContext(tc, sc);
            ProcessorContext p = mock(ProcessorContext.class);
            job.setCurrentProcessor(p);

            assertTrue(job.checkExecutionTime());

            for (int i = 0; i < jobWcet - 1; ++i) {
                assertFalse(job.updateExecution(i).isPresent());
            }

            Optional<TaskContextInterface> otc = job.updateExecution(0);
            assertTrue(otc.isPresent());
            assertTrue(tc.equals(otc.get()));
            assertFalse(p.getJob().isPresent());
            assertFalse(job.getCurrentProcessor().isPresent());
            assertFalse(job.checkExecutionTime());

            verify(p, times(1)).setJob(null);
            assertThrows(RuntimeException.class, () -> job.updateExecution(1));
        }
    }
}
