package mvd.jester.simulator.maiabertogna;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import java.util.Optional;
import java.util.concurrent.ThreadLocalRandom;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import mvd.jester.model.Segment;
import mvd.jester.simulator.internals.JobContextInterface;
import mvd.jester.simulator.internals.maiabertogna.SegmentContext;
import mvd.jester.simulator.internals.maiabertogna.TaskContext;
import mvd.jester.simulator.internals.ProcessorContext;

/**
 * TestSegmentContext
 */
public class TestSegmentContext {

    @Test
    @DisplayName("Check if segment returns the jobs correctly.")
    void testGetNextJob() {
        for (int run = 0; run < 100; ++run) {
            long numberOfJobs = ThreadLocalRandom.current().nextLong(1, 10);
            long jobWcet = ThreadLocalRandom.current().nextLong(20, 50);
            Segment s = new Segment(jobWcet, numberOfJobs);
            TaskContext tc = mock(TaskContext.class);
            when(tc.acceptNotification(jobWcet - 1)).thenReturn(Optional.empty());

            SegmentContext sc = new SegmentContext(tc, s);

            Optional<JobContextInterface> job = Optional.empty();
            for (int i = 0; i < numberOfJobs; ++i) {
                job = sc.getNextJob();
                assertTrue(job.isPresent());
                job.get().setCurrentProcessor(new ProcessorContext(i));
            }

            assertFalse(sc.getNextJob().isPresent());

            sc = new SegmentContext(tc, s);

            for (int i = 0; i < numberOfJobs; ++i) {
                job = sc.getNextJob();
                assertTrue(job.isPresent());
                job.get().setCurrentProcessor(new ProcessorContext(i));
                for (int j = 0; j < jobWcet; ++j) {
                    job.get().updateExecution(i);
                }
            }

            assertFalse(sc.getNextJob().isPresent());
        }
    }
}
